package info.archinnov.achilles.entity.parsing;

import static info.archinnov.achilles.entity.metadata.PropertyType.*;
import info.archinnov.achilles.entity.metadata.JoinProperties;
import info.archinnov.achilles.entity.metadata.PropertyMeta;
import info.archinnov.achilles.entity.parsing.context.PropertyParsingContext;
import java.lang.reflect.Field;
import java.util.Arrays;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * JoinPropertyParser
 * 
 * @author DuyHai DOAN
 * 
 */
public class JoinPropertyParser {
    private static final Logger log = LoggerFactory.getLogger(JoinPropertyParser.class);

    private PropertyFilter filter = new PropertyFilter();
    private PropertyParser parser = new PropertyParser();

    public PropertyMeta<?, ?> parseJoin(PropertyParsingContext context) {

        Class<?> entityClass = context.getCurrentEntityClass();
        Field field = context.getCurrentField();

        PropertyMeta<?, ?> joinPropertyMeta = this.parser.parse(context);

        log.debug("Parsing join property meta {} for entity {}", joinPropertyMeta.getPropertyName(), context
                .getCurrentEntityClass()
                .getCanonicalName());

        joinPropertyMeta.setJoinProperties(findCascadeType(entityClass.getCanonicalName(), field));

        // Override each type by their JOIN type counterpart
        switch (joinPropertyMeta.type()) {
            case SIMPLE:
            case LAZY_SIMPLE:
                joinPropertyMeta.setType(JOIN_SIMPLE);
                break;
            case LIST:
            case LAZY_LIST:
                joinPropertyMeta.setType(JOIN_LIST);
                break;
            case SET:
            case LAZY_SET:
                joinPropertyMeta.setType(JOIN_SET);
                break;
            case MAP:
            case LAZY_MAP:
                joinPropertyMeta.setType(JOIN_MAP);
                break;
            default:
                break;
        }

        context.getJoinPropertyMetaToBeFilled().put(joinPropertyMeta, joinPropertyMeta.getValueClass());

        return joinPropertyMeta;
    }

    private JoinProperties findCascadeType(String entityFQN, Field field) {
        log.trace("Find cascade type for property {} of entity class {}", field.getName(), field
                .getDeclaringClass()
                .getCanonicalName());

        JoinProperties joinProperties = new JoinProperties();

        if (filter.hasAnnotation(field, OneToOne.class)) {
            OneToOne oneToOne = field.getAnnotation(OneToOne.class);
            joinProperties.addCascadeType(Arrays.asList(oneToOne.cascade()));
        } else if (filter.hasAnnotation(field, OneToMany.class)) {
            OneToMany oneToMany = field.getAnnotation(OneToMany.class);
            joinProperties.addCascadeType(Arrays.asList(oneToMany.cascade()));
        }
        if (filter.hasAnnotation(field, ManyToOne.class)) {
            ManyToOne manyToOne = field.getAnnotation(ManyToOne.class);
            joinProperties.addCascadeType(Arrays.asList(manyToOne.cascade()));
        } else if (filter.hasAnnotation(field, ManyToMany.class)) {
            ManyToMany manyToMany = field.getAnnotation(ManyToMany.class);
            joinProperties.addCascadeType(Arrays.asList(manyToMany.cascade()));
        }

        log.trace("Built join properties for property {} of entity class {}", joinProperties, field.getName(), field
                .getDeclaringClass()
                .getCanonicalName());
        return joinProperties;
    }
}
